unit IniObject;

interface

uses Classes, Sysutils, TypInfo, Controls;

type
  TIniObject = class(TPersistent)
  protected
    FAll_Props, FBackup: array of record name, value: string end;
    procedure GetControl(W: TWinControl; Name: string; var Value: string);
    function GetParamByName(Name: string): variant;
    procedure SetParamByName(Name: string; const Value: variant);
  public
    procedure GetOrSetProps(SetProp: boolean);
    procedure Props2Array;
    procedure Array2Props;
    procedure LoadFromFile(FN: string);
    procedure SaveToFile(FN: string);
    procedure LoadFromControls(W: TWinControl);
    procedure SaveToControls(W: TWinControl);
    function Equal(W: TWinControl): boolean;
    procedure BackupProps;
    procedure RestoreProps;
    property ParamByName[Name: string]: variant read GetParamByName write SetParamByName;
  end;

implementation

{ TIniObject }

procedure TIniObject.Array2Props;
begin
  GetOrSetProps(true);
end;

procedure TIniObject.GetOrSetProps(SetProp: boolean);
  var I, Count, J: Integer;
      PropInfo: PPropInfo;
      PropList: PPropList;
      PropType: PTypeInfo;
begin
  if not SetProp then SetLength(FAll_Props, 0);
  Count := GetTypeData(Self.ClassInfo)^.PropCount;
  if Count > 0 then begin
    GetMem(PropList, Count * SizeOf(Pointer));
    try
      GetPropInfos(Self.ClassInfo, PropList);
      for I := 0 to Count - 1 do begin
        PropInfo := PropList^[I];
        if PropInfo = nil then break;
        PropType := PPropInfo(PropInfo)^.PropType^;

        if SetProp then begin
          for J := 0 to High(FAll_Props) do
            if UpperCase(FAll_Props[J].name) = UpperCase(PPropInfo(PropInfo)^.Name) then begin
              case PropType^.Kind of
                tkInteger, tkEnumeration:
                  SetOrdProp(Self, PropInfo, StrToInt(FAll_Props[J].value));
                tkString, tkLString, tkWString:
                  SetStrProp(Self, PropInfo, FAll_Props[J].value);
              end;
              break;
            end;
        end else begin
          SetLength(FAll_Props, Length(FAll_Props)+1);
          FAll_Props[High(FAll_Props)].name := UpperCase(PPropInfo(PropInfo)^.Name);
          case PropType^.Kind of
            tkInteger, tkEnumeration:
              FAll_Props[High(FAll_Props)].value := IntToStr(GetOrdProp(Self, PropInfo));
            tkString, tkLString, tkWString:
              FAll_Props[High(FAll_Props)].value := GetStrProp(Self, PropInfo);
          end;
        end;
      end;
    finally
      FreeMem(PropList, Count * SizeOf(Pointer));
    end;
  end;
end;

function IsClass(C: TClass; ParentClass: string): boolean;
begin
  Result := C.ClassName = ParentClass;
  if not Result and (C.ClassParent <> nil) then Result := IsClass(C.ClassParent, ParentClass);
end;

procedure TIniObject.GetControl(W: TWinControl; Name: string; var Value: string);
  var i: integer;
      C: TClass;
begin
  C := W.ClassType;

  if UpperCase(W.Name) = UpperCase(Name) then begin
    if IsClass(C, 'TsDecimalSpinEdit') then begin
      if GetOrdProp(W, 'DecimalPlaces') = 1 then Value := IntToStr(round(GetFloatProp(W, 'Value')*10))
                                            else Value := GetStrProp(W, 'Text')
    end else
    if IsClass(C, 'TsComboEdit') or IsClass(C, 'TsTimePicker') or IsClass(C, 'TsEdit') or IsClass(C, 'TsMaskEdit') then Value := GetStrProp(W, 'Text')
    else
    if IsClass(C, 'TsTrackBar') then Value := IntToStr(GetOrdProp(W, 'Position'))
    else
    if IsClass(C, 'TsCheckBox') then Value := IntToStr(GetOrdProp(W, 'Checked'))
    else
    if IsClass(C, 'TsMemo') then Value:= StringReplace(TStrings(GetObjectProp(W, 'Lines')).Text, #13#10, ';', [rfReplaceAll])
    else
    if IsClass(C, 'TsComboBox') then
      case GetOrdProp(W, 'Style') of //TComboBoxStyle = (csDropDown, csSimple, csDropDownList, csOwnerDrawFixed, csOwnerDrawVariable);
        2: Value := IntToStr(GetOrdProp(W, 'ItemIndex'));
        0: Value := GetStrProp(W, 'Text');
      end
    else
    if IsClass(C, 'TGroupBox') then begin
      for i := 0 to W.ControlCount-1 do
        if IsClass(W.Controls[i].ClassType,  'TsRadioButton') then
          if GetOrdProp(W.Controls[i], 'Checked') = 1 then Value := IntToStr(GetOrdProp(W.Controls[i], 'TabOrder'));
    end
  end else
    for i := 0 to W.ControlCount-1 do
      if W.Controls[i] is TWinControl then GetControl(W.Controls[i] as TWinControl, Name, Value);
end;

procedure TIniObject.LoadFromControls(W: TWinControl);
  var i: integer;
begin
  Props2Array;
  for i := 0 to High(FAll_Props) do
    GetControl(W, FAll_Props[i].name, FAll_Props[i].value);
  Array2Props;
end;

procedure TIniObject.LoadFromFile(FN: string);
  var s: TStringList;
      i, j: integer;
begin
  if FN = '' then FN := ChangeFileExt(ParamStr(0), '.ini');
  s := TStringList.Create;
  try
    s.LoadFromFile(FN);
    SetLength(FAll_Props, s.Count);
    for i := 0 to High(FAll_Props) do begin
      j := pos('=', s[i]);
      FAll_Props[i].name := Trim(UpperCase(copy(s[i], 1, j-1)));
      FAll_Props[i].value := Trim(copy(s[i], j+1, Length(s[i])));
    end;
    Array2Props;
  finally
    s.Free;
  end;
end;

procedure TIniObject.Props2Array;
begin
  GetOrSetProps(false);
end;

procedure TIniObject.SaveToControls(W: TWinControl);
                                                
  procedure SetControl(W: TWinControl; Name, Value: string);
    var i: integer;
        C: TClass;
  begin
    C := W.ClassType;
    if UpperCase(W.Name) = UpperCase(Name) then begin
      if IsClass(C, 'TsDecimalSpinEdit') then begin
        if GetOrdProp(W, 'DecimalPlaces') = 1 then SetFloatProp(W, 'Value', StrToInt(Value)/10)
                                              else SetStrProp(W, 'Text', Value)
      end else
      if IsClass(C, 'TsComboEdit') or IsClass(C, 'TsEdit') or IsClass(C, 'TsTimePicker') or IsClass(C, 'TsMaskEdit') then SetStrProp(W, 'Text', Value)
      else
      if IsClass(C, 'TsMemo') then TStrings(GetObjectProp(W, 'Lines')).Text := StringReplace(Value, ';', #13#10, [rfReplaceAll])
      else
      if IsClass(C, 'TsTrackBar') then SetOrdProp(W, 'Position', StrToInt(Value))
      else
      if IsClass(C, 'TsCheckBox') then SetOrdProp(W, 'Checked', StrToInt(Value) and 1)
      else
      if IsClass(C, 'TsComboBox') then
        case GetOrdProp(W, 'Style') of //TComboBoxStyle = (csDropDown, csSimple, csDropDownList, csOwnerDrawFixed, csOwnerDrawVariable);
          2: SetOrdProp(W, 'ItemIndex', StrToInt(Value));
          0: SetStrProp(W, 'Text', Value);
        end
      else
      if IsClass(C, 'TGroupBox') then begin
        for i := 0 to W.ControlCount-1 do
          if IsClass(W.Controls[i].ClassType,  'TsRadioButton') then
            if GetOrdProp(W.Controls[i], 'TabOrder') = StrToInt(Value) then SetOrdProp(W.Controls[i], 'Checked', 1);
      end;
    end else
      for i := 0 to W.ControlCount-1 do
        if W.Controls[i] is TWinControl then SetControl(W.Controls[i] as TWinControl, Name, Value)
  end;

  var i: integer;
begin
  Props2Array;
  for i := 0 to High(FAll_Props) do
    SetControl(W, FAll_Props[i].name, FAll_Props[i].value);
end;

procedure TIniObject.SaveToFile(FN: string);
  var s: TStringList;
      i: integer;
begin
  if FN = '' then FN := ChangeFileExt(ParamStr(0), '.ini');
  Props2Array;
  s := TStringList.Create;
  try
    for i := 0 to High(FAll_Props) do
      s.Add(FAll_Props[i].name + '=' + FAll_Props[i].value);
    s.SaveToFile(FN);
  finally
    s.Free;
  end;
end;

function TIniObject.Equal(W: TWinControl): boolean;
  var i: integer;
      name, value: string;
begin
  Props2Array;
  Result := true;
  for i := 0 to High(FAll_Props) do begin
    name  := FAll_Props[i].name;
    value  := FAll_Props[i].value;
    GetControl(W, name, value);
    if FAll_Props[i].value <> value then begin
      Result := false;
      break;
    end;
  end;
end;

procedure TIniObject.BackupProps;
  var i: integer;
begin
  Props2Array;
  SetLength(FBackup, Length(FAll_Props));
  for i := 0 to High(FAll_Props) do
    FBackup[i] := FAll_Props[i];
end;

procedure TIniObject.RestoreProps;
  var i: integer;
begin
  for i := 0 to High(FAll_Props) do
    FAll_Props[i] := FBackup[i];
  SetLength(FBackup, 0);
  Array2Props;
end;

function TIniObject.GetParamByName(Name: string): variant;
begin
  try
    Result := GetPropValue(Self, Name, false);
  except
    Result := 'param not found';
  end;
end;

procedure TIniObject.SetParamByName(Name: string; const Value: variant);
begin
  try
    SetPropValue(Self, Name, Value)
  except
  end;
end;

end.
